@@ -38,6 +38,9 @@ void FactorInt(FacVec& fvec, long n)
 {
    if (n <= 1) Error("internal error: FactorInt(FacVec,long n) with n<=1");
 
+   if (NTL_OVERFLOW(n, 1, 0))
+      Error("internal error: FactorInt(FacVec,long n) with n too large");
+
    long NumFactors;
    long q;
 
