@@ -5,72 +5,53 @@
 
 NTL_START_IMPL
 
-const long MaxAllocBlock = 10000;
-
 
 void GF2XVec::SetSize(long n, long d)
 {
    if (n < 0 || d <= 0) Error("bad args to GF2XVec::SetSize()");
+
    if (v)
-      Error("GF2XVec initialized more than once");
+      Error("illegal GF2XVec initialization");
 
-   if (d >= (1L << (NTL_BITS_PER_LONG-4))/NTL_BITS_PER_LONG)
-      Error("size too big in GF2XVec::SetSize");
-
-   if (n >= long((1L << (NTL_BITS_PER_LONG-4))/sizeof(GF2X)))
-      Error("length too big in GF2XVec::SetSize");
+   len = n;
+   bsize = d;
 
    if (n == 0) return;
 
-   long size = (d+2);
-   long AllocAmt = MaxAllocBlock / size;
-   if (AllocAmt == 0) AllocAmt = 1;
-
-   v = (GF2X*) malloc(n * (sizeof (GF2X)));
+   v = (GF2X*) NTL_MALLOC(n, sizeof(GF2X), 0);
    if (!v) Error("out of memory in GF2XVec::SetSize()");
 
    long i = 0;
    long m;
-   _ntl_ulong *p, *q;
    long j;
 
    while (i < n) {
-      m = min((n-i), AllocAmt);
-      p = (_ntl_ulong *) malloc(m*size*(sizeof (_ntl_ulong)));
-      if (!p) Error("out of memory in GF2XVec::SetSize()");
-      for (j = 0, q = p+2; j < m; j++, i++, q += size) {
-         q[-2] = (d << 1) | 1;
-         q[-1] = 0;
-         v[i].xrep.rep = q; 
-      }
+      m = WV_BlockConstructAlloc(v[i].xrep, d, n-i);
+      for (j = 1; j < m; j++)
+         WV_BlockConstructSet(v[i].xrep, v[i+j].xrep, j);
+      i += m;
    }
-
-   len = n;
-   bsize = d;
 }
 
+
 void GF2XVec::kill()
 {
    long n = len;
-   long d = bsize;
-   if (n == 0) return;
 
-   long size = (d+2);
-   long AllocAmt = MaxAllocBlock / size;
-   if (AllocAmt == 0) AllocAmt = 1;
+   len = 0; bsize = 0;
+
+   if (n == 0) return;
 
    long i = 0;
    long m;
 
    while (i < n) {
-      m = min((n-i), AllocAmt);
-      free(v[i].xrep.rep-2);
+      m = WV_BlockDestroy(v[i].xrep);
       i += m;
    }
 
    free(v);
-
-   v = 0; len = 0; bsize = 0;
+   v = 0; 
 }
 
 
