@@ -265,6 +265,8 @@ void RandomBasisElt(ZZ_pX& g, const vec_long& D, const vec_ZZVec& M)
          conv(v[j], t1);
       }
    }
+
+   g.normalize();
 }
 
 
@@ -1434,7 +1436,7 @@ void BuildIrred(ZZ_pX& f, long n)
    if (n <= 0)
       Error("BuildIrred: n must be positive");
 
-   if (n >= (1L << (NTL_BITS_PER_LONG-4))) Error("overflow in BuildIrred");
+   if (NTL_OVERFLOW(n, 1, 0)) Error("overflow in BuildIrred");
 
    if (n == 1) {
       SetX(f);
