@@ -920,11 +920,11 @@ void InnerProduct(zz_pX& x, const vec_zz_p& v, long low, long high,
       zz_p w = (v[i]);
 
       long W = rep(w);
-      double Wpinv = ((double) W)*pinv;
+      mulmod_precon_t Wpinv = PrepMulModPrecon(W, p, pinv); // ((double) W)*pinv;
       const zz_p *hp = h.elts();
 
       for (j = 0; j < m; j++) {
-         long S = MulMod2(rep(hp[j]), W, p, Wpinv);
+         long S = MulModPrecon(rep(hp[j]), W, p, Wpinv);
          S = AddMod(S, rep(tp[j]), p);
          tp[j].LoopHole() = S;
       }
@@ -1625,7 +1625,7 @@ void TraceMod(zz_p& x, const zz_pX& a, const zz_pXModulus& F)
       Error("trace: bad args");
 
    if (F.tracevec.length() == 0) 
-      TraceVec(F);
+      ComputeTraceVec(F);
 
    InnerProduct(x, a.rep, F.tracevec);
 }
